#ifndef _AFTER_H
#  include "after.h"
#endif
#ifndef _CDI_H
#  include "cdi.h"
#endif

void FreeSpectral(struct Control *All)
{
  int code;

  for ( code = MaxCodes-1; code >= 0; --code )
    if ( All[code].spectral )
      All[code].spectral = FreeMemory(All[code].spectral);
}

void FreeFourier(struct Control *All)
{
  int code;

  for ( code = 0; code < MaxCodes; code++ )
    if ( All[code].fourier )
      All[code].fourier = FreeMemory(All[code].fourier);
}

void FreeHybrid(struct Control *All)
{
  int code;

  for ( code = 0; code < MaxCodes; code++ )
    if ( All[code].hybrid )
      All[code].hybrid = FreeMemory(All[code].hybrid);
}

void FreeGrid(struct Control *All)
{
  int code;

  for ( code = 0; code < MaxCodes; code++ )
    if ( All[code].grid )
      All[code].grid = FreeMemory(All[code].grid);
}

void FreeMean(struct Control *All)
{
  int code;

  for ( code = 0; code < MaxCodes; code++ )
    if ( All[code].mean )
      All[code].mean = FreeMemory(All[code].mean);
}

void FreeSamp(struct Control *All)
{
  int code;

  for ( code = 0; code < MaxCodes; code++ )
    if ( All[code].samp )
      All[code].samp = FreeMemory(All[code].samp);
}

void printCodes(void)
{
  int code, i;
  int tableID = tableInq(-1, 128, "echam4");
  int ncodes;
  int codes[] = {34,35,36,131,132,135,148,149,151,156,157,259,260,261,262,263,264,268,269,270,271,275};
  char *name, *longname;

  ncodes = sizeof(codes)/sizeof(codes[0]);

  lprintf(stdout);

  fprintf(stdout, "  Code Name              Longname\n");
  fprintf(stdout, "  ---- ----              --------\n");

  for ( i = 0; i < ncodes; i++ )
    {
      code     = codes[i];
      name     = tableInqParNamePtr(tableID, code);
      longname = tableInqParLongnamePtr(tableID, code);

      fprintf(stdout, " %4d", code);
      if ( name == NULL )
	fprintf(stdout, "  var%d", code);
      else
	{
	  fprintf(stdout, "  %-16s", name);
	  if ( longname != NULL )
	    fprintf(stdout, "  %s", longname);
	}
      fprintf(stdout, "\n");
    }

  lprintf(stdout);
}


void CheckContent(struct Control *All, int timestep)
{
  int code;
  extern int labort;
  /*  extern int NumLevelRequest; */

  for ( code = 0; code < 272; code++ )
    {
      /*  if ( code == GEOPOTENTIAL ) continue; */
      if ( code ==          SLP ) continue;
      if ( code == GEOPOTHEIGHT ) continue;
      if ( code ==       STREAM ) continue;
      if ( code ==      VELOPOT ) continue;
      if ( code ==       U_WIND ) continue;
      if ( code ==       V_WIND ) continue;
      if ( code ==        OMEGA ) continue;
      if ( code ==    RHUMIDITY ) continue;
      if ( code ==    LOW_CLOUD ) continue;
      if ( code ==    MID_CLOUD ) continue;
      if ( code ==    HIH_CLOUD ) continue;
      if ( code ==           PS ) continue;
      if ( code ==     HUMIDITY )
	{
	  if ( All[code].needed && !All[code].selected &&
	       All[code].spectral == NULL &&
	       All[code].hybrid   == NULL )
	    {
	      Warning( "No humidity in data file, set to zero !");
	      All[code].needed = FALSE;
	    }
	}
      else
	{
	  if ( All[code].needed && !All[code].comp &&
	       All[code].spectral == NULL &&
	       All[code].hybrid   == NULL )
	    {
	      if ( labort )
		Error( "Code  %3d not found at timestep %d!", code, timestep);
	      else
		Warning( "Code  %3d not found at timestep %d!", code, timestep);
	    }
	}
    }
  /*
  if ( NumLevelRequest > 0 )
    {
      All[HALF_PRESS].needed = 1;
      All[FULL_PRESS].needed = 1;
    }

  code = HALF_PRESS;
  if ( All[code].needed && !All[code].comp &&
       All[code].spectral == NULL && All[code].hybrid == NULL )
    Error( "Hybrid model level not found!");

  code = FULL_PRESS;
  if ( All[code].needed && !All[code].comp &&
       All[code].spectral == NULL && All[code].hybrid == NULL )
    Error( "Hybrid model level not found!");
  */
}


void CheckAnalyses(struct Control *All)
{
  int code;
  extern int labort;

  for ( code = 0; code < 272; code++ )
    if ( All[code].needed      && 
	 code != DIVERGENCE    &&
	 code != VORTICITY     &&
	 code != STREAM        &&
	 code != U_WIND        &&
	 code != HUMIDITY      &&
	 code != VELOPOT       &&
	 code != V_WIND        &&
	 code != RHUMIDITY     &&
	 code != GEOPOTHEIGHT  && 
	 code != PS            &&
         All[code].spectral == NULL && All[code].grid == NULL )
      {
	if ( labort )
	  Error( "Code  %3d not found", code);
	else
	  Warning( "Code  %3d not found", code);
      }
}


void MakeDependencies(struct Control *All, int varcode, int depcode)
{
  extern int Debug;

  if ( All[varcode].needed && ! All[varcode].detected )
    {
      All[depcode].needed = TRUE;
      All[varcode].comp   = TRUE;

      if ( Debug )
	fprintf(stderr, "Needed code %d to compute code %d\n", depcode, varcode);

      if ( All[depcode].ivarID == -1 )
	{
	  if ( depcode == U_WIND )
	    {
	      MakeDependencies(All, U_WIND, DIVERGENCE);
	      MakeDependencies(All, U_WIND, VORTICITY);
	    }
	  if ( depcode == V_WIND )
	    {
	      MakeDependencies(All, V_WIND, DIVERGENCE);
	      MakeDependencies(All, V_WIND, VORTICITY);
	    }
	}

      if ( All[varcode].ivarID == -1 )
	{
	  if ( All[depcode].ivarID == -1 )
	    {
	      Error( "code %d undefined, needed to compute code %d", depcode, varcode);
	    }
	  else
	    {
	      All[varcode].ivarID   = All[depcode].ivarID;
	      All[varcode].igridID  = All[depcode].igridID;
	      All[varcode].ogridID  = All[depcode].ogridID;
	      All[varcode].izaxisID = All[depcode].izaxisID;
	      All[varcode].ozaxisID = All[depcode].ozaxisID;
	    }
	}
    }
}

void CheckDependencies(struct Control *All, int analysisdata)
{
  MakeDependencies(All, VELOPOT, U_WIND);
  MakeDependencies(All, VELOPOT, V_WIND);
  MakeDependencies(All, VELOPOT, VORTICITY);
  MakeDependencies(All, VELOPOT, DIVERGENCE);

  MakeDependencies(All, STREAM, U_WIND);
  MakeDependencies(All, STREAM, V_WIND);
  MakeDependencies(All, STREAM, VORTICITY);
  MakeDependencies(All, STREAM, DIVERGENCE);

  MakeDependencies(All, VORTICITY, U_WIND);
  MakeDependencies(All, VORTICITY, V_WIND);

  MakeDependencies(All, DIVERGENCE, U_WIND);
  MakeDependencies(All, DIVERGENCE, V_WIND);

  MakeDependencies(All, U_WIND, VORTICITY);
  MakeDependencies(All, U_WIND, DIVERGENCE);
  MakeDependencies(All, U_WIND, V_WIND);

  MakeDependencies(All, V_WIND, VORTICITY);
  MakeDependencies(All, V_WIND, DIVERGENCE);
  MakeDependencies(All, V_WIND, U_WIND);

  MakeDependencies(All, WINDSPEED, U_WIND);
  MakeDependencies(All, WINDSPEED, V_WIND);

  if ( analysisdata )
    {
      MakeDependencies(All, RHUMIDITY, HUMIDITY);
      MakeDependencies(All, RHUMIDITY, TEMPERATURE);
      MakeDependencies(All, HUMIDITY, RHUMIDITY);
      MakeDependencies(All, HUMIDITY, TEMPERATURE);
      MakeDependencies(All, GEOPOTHEIGHT, GEOPOTENTIAL);
    }
  else
    {
      MakeDependencies(All, THETAF, TEMPERATURE);
      MakeDependencies(All, SLP, TEMPERATURE);
    }

  MakeDependencies(All, SW_ATM, 176);
  MakeDependencies(All, SW_ATM, 178);
  MakeDependencies(All, LW_ATM, 177);
  MakeDependencies(All, LW_ATM, 179);
  MakeDependencies(All, NET_ATM, 176);
  MakeDependencies(All, NET_ATM, 177);
  MakeDependencies(All, NET_ATM, 178);
  MakeDependencies(All, NET_ATM, 179);

  MakeDependencies(All, SW_BOT_CLF, 176);
  MakeDependencies(All, SW_BOT_CLF, 185);
  MakeDependencies(All, LW_BOT_CLF, 177);
  MakeDependencies(All, LW_BOT_CLF, 186);
  MakeDependencies(All, SW_TOP_CLF, 178);
  MakeDependencies(All, SW_TOP_CLF, 187);
  MakeDependencies(All, LW_TOP_CLF, 179);
  MakeDependencies(All, LW_TOP_CLF, 188);
  MakeDependencies(All, NET_TOP_CLF, 178);
  MakeDependencies(All, NET_TOP_CLF, 179);
  MakeDependencies(All, NET_TOP_CLF, 187);
  MakeDependencies(All, NET_TOP_CLF, 188);

  MakeDependencies(All, ALL_WATER, 222);
  MakeDependencies(All, LOW_WATER, 222);
  MakeDependencies(All, MID_WATER, 222);
  MakeDependencies(All, HIH_WATER, 222);

  MakeDependencies(All, LOW_CLOUD, 223);
  MakeDependencies(All, MID_CLOUD, 223);
  MakeDependencies(All, HIH_CLOUD, 223);

  if ( All[LOW_CLOUD].comp || All[MID_CLOUD].comp || All[HIH_CLOUD].comp )
    {
      static int zaxisID = -999;
      if ( zaxisID == -999 ) zaxisID = zaxisCreate(ZAXIS_SURFACE, 1);

      All[LOW_CLOUD].izaxisID = zaxisID;
      All[LOW_CLOUD].ozaxisID = zaxisID;
      All[MID_CLOUD].izaxisID = zaxisID;
      All[MID_CLOUD].ozaxisID = zaxisID;
      All[HIH_CLOUD].izaxisID = zaxisID;
      All[HIH_CLOUD].ozaxisID = zaxisID;
    }
}

void AnalysisDependencies(struct Control *All, int ncodes)
{
   int code;

   for ( code = 0; code < ncodes; code++ )
     All[code].needed = All[code].selected;

   MakeDependencies(All, PS, LNPS);

   CheckDependencies(All, 1);
}

void EchamDependencies(struct Control *All, int ncodes, int type, int source)
{
  int code;

  for ( code = 0; code < ncodes; code++ )
    All[code].needed = All[code].selected;

  for ( code = 0; code < ncodes; code++ )
    if ( All[code].detected == FALSE ) All[code].ivarID = -1;

  if ( type >= 50 )
    {
      All[U_WIND].needed |= All[DIVERGENCE].needed;
      All[V_WIND].needed |= All[DIVERGENCE].needed;
      All[U_WIND].needed |=  All[VORTICITY].needed;
      All[V_WIND].needed |=  All[VORTICITY].needed;
      All[U_WIND].needed |=    All[VELOPOT].needed;
      All[V_WIND].needed |=    All[VELOPOT].needed;
      All[U_WIND].needed |=     All[STREAM].needed;
      All[V_WIND].needed |=     All[STREAM].needed;
    }

  if ( type >= 30 ) All[LNPS].needed = TRUE;

  if ( type >= 20 )
    {
      MakeDependencies(All, THETAF, LNPS);
      MakeDependencies(All, SLP, LNPS);
      MakeDependencies(All, SLP, GEOPOTENTIAL);
      /*
      MakeDependencies(All, SLP, HALF_PRESS);
      MakeDependencies(All, SLP, FULL_PRESS);
      */
      MakeDependencies(All, RHUMIDITY, TEMPERATURE);
      MakeDependencies(All, RHUMIDITY, HUMIDITY);
      MakeDependencies(All, RHUMIDITY, LNPS);
      MakeDependencies(All, GEOPOTHEIGHT, TEMPERATURE);
      MakeDependencies(All, GEOPOTHEIGHT, HUMIDITY);
      MakeDependencies(All, GEOPOTHEIGHT, LNPS);
      MakeDependencies(All, OMEGA, DIVERGENCE);
      MakeDependencies(All, OMEGA, U_WIND);
      MakeDependencies(All, OMEGA, V_WIND);
      MakeDependencies(All, OMEGA, LNPS);
      MakeDependencies(All, OMEGA, DPSDX);
      MakeDependencies(All, OMEGA, DPSDY);
    }

   MakeDependencies(All, DPSDX, LNPS);
   MakeDependencies(All, HALF_PRESS, LNPS);
   MakeDependencies(All, PS, LNPS);

   MakeDependencies(All, SURF_RUNOFF, 142);
   MakeDependencies(All, SURF_RUNOFF, 143);
   MakeDependencies(All, SURF_RUNOFF, 182);
   MakeDependencies(All, SURF_RUNOFF, 221);  /* snow depth change */

   MakeDependencies(All, THETAF, TS);

   MakeDependencies(All, FRESH_WATER, 142);
   MakeDependencies(All, FRESH_WATER, 143);
   MakeDependencies(All, FRESH_WATER, 182);

   MakeDependencies(All, PRECIP, 142);
   MakeDependencies(All, PRECIP, 143);

   if ( source != S_ECHAM5 )
     {
       MakeDependencies(All, NET_WATER, 142);
       MakeDependencies(All, NET_WATER, 143);
       MakeDependencies(All, NET_WATER, 160);
       MakeDependencies(All, NET_WATER, 182);

       MakeDependencies(All, NET_TOP, 178);
       MakeDependencies(All, NET_TOP, 179);

       MakeDependencies(All, NET_BOT, 176);
       MakeDependencies(All, NET_BOT, 177);

       MakeDependencies(All, NET_HEAT, 146);
       MakeDependencies(All, NET_HEAT, 147);
       MakeDependencies(All, NET_HEAT, 176);
       MakeDependencies(All, NET_HEAT, 177);
       MakeDependencies(All, NET_HEAT, 218);
       /*
	 if ( source == S_ECHAM5 )
	 {
	 MakeDependencies(All, NET_HEAT, 206);
	 MakeDependencies(All, NET_HEAT, 208);
	 MakeDependencies(All, NET_HEAT, 209);
	 }
	 else
       */
       {
	 MakeDependencies(All, NET_HEAT, 220);
       }
     }

   MakeDependencies(All, SW_CLF, 178);
   MakeDependencies(All, SW_CLF, 224);

   MakeDependencies(All, LW_CLF, 179);
   MakeDependencies(All, LW_CLF, 225);

   MakeDependencies(All, NET_CLF, 178);
   MakeDependencies(All, NET_CLF, 179);
   MakeDependencies(All, NET_CLF, 224);
   MakeDependencies(All, NET_CLF, 225);

   if ( All[DPSDX].needed || All[DPSDY].needed || 
	All[GEOPOTHEIGHT].comp || All[SLP].comp || All[THETAF].needed ||
        All[HALF_PRESS].needed || All[RHUMIDITY].comp || All[OMEGA].comp ||
        type >= 30 )
     All[PS_PROG].comp = TRUE;

   CheckDependencies(All, 0);
}
